/*
  Determine CPU Vendor & Stepping.
*/

inline void cpuid(unsigned int op, unsigned int *eax, unsigned int *ebx, unsigned int *ecx, unsigned int *edx);
void regprint(FILE *fp, unsigned int preg);
void identify_CPU(void);

#include "cpu_intel.c"
#include "cpu_amd.c"


inline void cpuid(unsigned int op, unsigned int *eax, unsigned int *ebx, unsigned int *ecx, unsigned int *edx)
{
    __asm__("cpuid"
	    : "=a" (*eax),
	      "=b" (*ebx),
	      "=c" (*ecx),
	      "=d" (*edx)
	    : "a" (op)
	    : "cc");
}


void regprint(FILE *fp, unsigned int preg)
{
    char regch[]="----";

    int i;
    for (i=0;i<4;i++) {
	regch[i]=(preg>>(8*i));
    }
    fprintf(fp, ((char *) regch));
}


void identify_CPU(void)
{
    FILE *logfile=0;
    unsigned int i,maxi;
    unsigned int eax,ebx,ecx,edx;

    /* Dump all the CPUID results in raw hex */
    cpuid(0, &eax,&ebx,&ecx,&edx);
    maxi = eax;
    for(i=0;i<=maxi;i++){
	cpuid(i, &eax,&ebx,&ecx,&edx);
    }

  /* Vendor ID and max CPUID level supported */
    cpuid(0, &eax,&ebx,&ecx,&edx);

    if (open_log_file(&logfile)==TRUE) {
	fprintf(logfile, "\tVendor ID: ");
	regprint(logfile, ebx);
	regprint(logfile, edx);
	regprint(logfile, ecx);
	fprintf(logfile, "\n");
	close_log_file(&logfile);
    }

    switch(ebx){

    case 0x756e6547:	/* 'uneG' Intel */
	Decode_Intel();
	break;

    case 0x68747541:	/* 'htuA' AMD */
	Decode_AMD();
	break;

    case 0x69727943:	/* 'iryC' Cyrix */
//	Decode_Cyrix();
	break;

    case 0x746e6543:	/* 'tneC' Centaur-Hauls */
//	Decode_IDT();
	break;

    case 0x20434d55:	/* ' CMU' UMC */
//	Decode_UMC();
	break;

    case 0x4778654e:	/* 'GxeN' NexGen */
//	Decode_NexGen();
	break;

    default:
	if (open_log_file(&logfile)==TRUE) {
	    fprintf(logfile, "Unknown vendor:");
	    regprint(logfile, ebx);
	    close_log_file(&logfile);
	}
	break;
    }
}
