/*	$Id: powertweak.c,v 0.1.0 1999/6/27 18:04:23 dj Exp $
 *
 *	Linux PCI Utilities -- Tune PCI devices.
 *
 *	Copyright (c) 1997--1999 Dave Jones <dave@powertweak.com>
 *	contains some code by Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 *
 *	Can be freely distributed and used under the terms of the GNU GPL.
 *	*NB* No code or data-tables from Powertweak may be used in any profit
 *	     making application (regardless of OS) without express permission.
 *
 * Note to purists, yes there are quite a few goto's used here, I tried to
 *  implement a clean solution without them, but it ended up being even more of
 *  a mess, and a lot harder to understand/debug. If you can do a better job
 *  than I did, go ahead. If I like it, I'll use it. -DJ
 *
 * TS: 4
 */
void (decode_if(char **fptr));

#include <ctype.h>
#include "parse_utils.c"
#include "poke.c"

void (decode_if(char **fptr))
{
    unsigned int iftest_reg, iftest_bit, iftest_value, iftest_actual;
    unsigned int fi_expected=1;

    const char if_string[]="if";
    const char fi_string[]="fi";
    const char if_sep[]="h:";		/*seperator*/

//    printf("Decoding if.\n");
    (*fptr)+=3; /* Skip 'if ' */

    iftest_reg = strtoul(*fptr, fptr+2, 16);
    (*fptr)+=2; /* Skip reg number */

    if (strncasecmp(*fptr, if_sep, 2)!=0)
	corrupt_configfile('h', **fptr);
    (*fptr)+=2;

    iftest_bit = strtoul(*fptr, fptr+1, 16);
    (*fptr)++; /* Skip bit number*/

    if (**fptr!='=')
	corrupt_configfile('=', **fptr);
    (*fptr)++;

    iftest_value = strtoul(*fptr, fptr+1, 16);
    (*fptr)++; /* Skip expected bit value*/

    /* Now we read the value, and compare it against the expected value */
    iftest_actual = pci_read_byte(dev, iftest_reg);

/*
    printf ("!IF reg:%x ", iftest_reg);
    printf("Bit:%x=", iftest_bit);
    printf("Value:%x ", iftest_value);
    printf("(Current=%x) ", iftest_actual);
    printf ("%x\n",iftest_actual & (1<<iftest_bit));
*/

    if ((iftest_actual & (1<<iftest_bit)) != (iftest_value<<iftest_bit)) {

keep_skipping:
	while (**fptr!='!' && **fptr!='#') {
	    (*fptr)++;
	}

	if ((**fptr) == '#') {
	    SkipComment(fptr);
	    goto keep_skipping;
	}

	(*fptr)++;	/* Skip the ! */
	
	if ((**fptr)=='.') {
	    corrupt_configfile('!', **fptr);
	}

	if (strncasecmp(*fptr, if_string, 2)==0) {
	    fi_expected++;
	    (*fptr)+=2; 	// Skip if
	    goto keep_skipping;
	}

	if (strncasecmp(*fptr, fi_string, 2)==0) {
	    fi_expected--;
	    (*fptr)+=2;		// Skip fi
	    if (fi_expected != 0)
		goto keep_skipping;
	}
    }
}




/*
 Here's where the magic happens.
 This routine gets passed a pointer to the part of the config file which
 pertains information relating to the device currently being examined.
 We read each register section and set it accordingly.
*/

void tune_device(char *configfile)
{
    unsigned int poke_reg, poke_high_bit, poke_low_bit, poke_value, current_value, i;
    char defaultchoicestxt[]="disabled,enabled\n";

    unsigned int c_offset=0, c_range=FALSE, c_bitmask=0;
    char *reasonptr;
    char *choiceptr;

    char nextchar;

    /*********************************************************************************
    **	Check for conditionals before R: fields.
    **********************************************************************************/

    while ((*configfile !='R') && (*configfile !='!')) {
	configfile++;
    }

    if (*configfile == '!') {
	configfile++;
	decode_special(&configfile);
    }
    /*********************************************************************************
    **	Get the R: field.
    **********************************************************************************/

    FindTag ('R', &configfile);

re_enter_reg:

    poke_reg = strtoul(configfile, &configfile+2, 16);
    configfile+=2;
    if (*configfile != 'h')		/* Make sure the register has a trailing h */
	corrupt_configfile('h', *configfile);
    configfile++;


    /*********************************************************************************
    **	Get the N: field.
    **********************************************************************************/

    FindTag ('N', &configfile);

re_enter_name:

    reasonptr=configfile;

    while (*configfile !='\n') {
    	configfile++;
    }

    current_value = pci_read_byte(dev, poke_reg);

    /*********************************************************************************
    **	Get the B: field.
    **********************************************************************************/

    FindTag ('B', &configfile);

    /* If we get here, we are at a B: field. */
    poke_high_bit = strtoul(configfile, &configfile+1, 16);

    configfile++;	/* Skip the high bit number */

    /* Now we check if this is a range B:?-? */
    if (*configfile=='-') {
	configfile++;	/* Skip the - */
        poke_low_bit = strtoul(configfile, &configfile+1, 16);
	configfile++;	/* Skip the low bit number */
    } else {
	poke_low_bit = poke_high_bit;
    }


    /*********************************************************************************
    **	Get the C: field.
    **********************************************************************************/

    nextchar = GetNextTag (&configfile);

    if (nextchar == 'C') {
	configfile+=2;	/* Skip the C: */
	c_range=FALSE;
	if (*configfile=='*')
	    c_range=TRUE;
	choiceptr=configfile;
	while (*configfile!='\n') {
    	    configfile++;
	}
    } else {
    /* No choice tag.  Use defaults. */
	choiceptr=defaultchoicestxt;
    }


    /*********************************************************************************
    **	Get the P: field.
    ** First we check if the GetNextTag() from the C: handler has advanced us to the P:
    **********************************************************************************/

    if (nextchar!='P') {
	FindTag ('P', &configfile);
    } else {
	/* Getting here means we had no C: tag and the GetNextTag from the C:
	 handler already advanced us. */
	configfile+=2;	/* Skip the P: */
    }

    current_value = pci_read_byte(dev, poke_reg);


//    if (*configfile=='?') {
//	printf("!?\n");
//	printf("%s\n", reason);
//        printf("POKE %xh,\n",poke_reg);
//        printf("bits%d-%d\t", poke_high_bit,poke_low_bit);
//	printf("Currently:%x\n", current_value);
//	exit(EXIT_FAILURE);
//    }

    if (c_range==TRUE) {
	poke_value=0;
	while(isdigit(*configfile)==TRUE) {
	    poke_value=poke_value<<4;
	    poke_value=poke_value | (strtoul(configfile, &configfile+1, 16));
	    configfile++;
	}

	c_bitmask=0;
	for (i=poke_low_bit;i<poke_high_bit;i++) {
	    c_bitmask|=1<<i;
	}
	poke_value=poke_value & c_bitmask;
	
    } else {

	poke_value=current_value;

	if (poke_high_bit == poke_low_bit) {

	    i= strtoul(configfile, &configfile+1, 16);
	    c_offset=i;

	    if (i==0)
		poke_value=(poke_value & (0xff ^ 1<<poke_high_bit));
	    else
		poke_value=(poke_value | (1<<poke_high_bit));

        } else {

	    c_offset=0;

	    for (i=poke_high_bit;i<=poke_low_bit;i--) {
		switch (*configfile) {

		    case '0':	poke_value=poke_value & (0xff ^ 1<<i);	/* Set to zero */
			        break;

		    case '1':	poke_value=poke_value | (1<<i);		/* Set to one */
				c_offset=c_offset|(1<<i);
				break;

		    case '?':	printf("!! ??'s in multichoice poke !!\n");
				exit(EXIT_FAILURE);
				break;
	    
		    default :	corrupt_configfile('_', *configfile);
				break;
		}
	    }
	}
	configfile++;	/* Skip the high bit number */
    }

/*
    printf("%s\t", reason);
    printf("POKE %xh,%x\t",poke_reg,poke_value);
    if (poke_high_bit==poke_low_bit)\
	printf("bit %d\t", poke_high_bit);
    else
	printf("bits %d-%d\t", poke_high_bit,poke_low_bit);
    if (poke_value != current_value)
	printf("Currently:%x", current_value);
    printf("\n");
*/
    /*********************************************************************************
    **	We now have all the information we need, so we do the actual poking.
    **********************************************************************************/

    /*		R:	N:			P:		C:	C:&P: */
    poke(poke_reg, reasonptr, current_value, poke_value, choiceptr, c_offset);


    /*********************************************************************************
    **	The poking is now done.
    **  Now we check whether we have other bits to set in the same register.
    **  If not, we skip to the next register, or the next device if no more regs
    **  remain to be tweaked.
    **********************************************************************************/

next:
    while ( (*configfile != '}') &&
	    (*configfile != '#') &&
	    (*configfile != '!') &&
	    (*configfile != 'N') &&
	    (*configfile != 'R') ) {
	configfile++;
    }

    switch (*configfile) {
    
	case '}' :		break;

	case '#' :		while(*configfile != '\n') {	/* Skip to EOL.*/
				    configfile++;
			    	}
				configfile++;
				goto next;

	case '!' :		configfile++;
				if (*configfile=='.')
				    goto out;
				decode_special(&configfile);
				goto next;
				

	case 'N' :		configfile++;
				if (*configfile==':') {
				    configfile++;
				    goto re_enter_name;
				} else {
				    corrupt_configfile(':', *configfile);
				}
				break;

	case 'R' :		configfile++;
				if (*configfile==':') {
				    configfile++;
				    goto re_enter_reg;
				} else {
				    corrupt_configfile(':', *configfile);
				}
				break;
    }

out:
    /* All registers for this device now tuned. */

/*    if (verbose>1 && ChangesMade==TRUE)
	printf("\n");
    printf("\n");
*/
}



//    printf("%s\n%x\n", choiceptr, c_offset);

    /* Copy the reason to buffer. */
/*    i=0;
    while (*reasonptr!='\n') {
	reason[i]=(*reasonptr)++;
	fprintf(stderr, "%c",reason[i]);
	i++;
    }
    reason[i++]='\0';

    printf("Copied reason\"%s\"", reason);

*/

    /* Copy the selected choice to buffer. */
/*    if (c_offset!=0) {
	for (i=0;i<c_offset;i++) {
	    while (*choiceptr!=',') {
		choiceptr++;
	    }
	}
    }
    i=0;
    while (*choiceptr!=',' && *choiceptr!='\n') {
	choice[i++]=(*choiceptr)++;
    }
    choice[i++]='\0';
*/

    /* Do the poking. */
/*    if (current_value == poke_value) {
	printf("\t %s: %s [0x%x,0x%x]\n", reason, choice, poke_reg, poke_value);
    } else {
	printf("\t %s: %s [0x%x,0x%x] -> [0x%x,0x%x]\n",
	    reason, choice,
	    poke_reg, current_value,
	    poke_reg, poke_value);
	//pci_write_byte (dev, poke_reg, poke_value);
    }
*/
