/*	$Id: powertweak.c,v 0.1.0 1999/6/27 18:02:33 dj Exp $
 *
 *	Linux PCI Utilities -- Tune PCI devices.
 *
 *	Copyright (c) 1997--1999 Dave Jones <dave@powertweak.com>
 *	contains some code by Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 *
 *	Can be freely distributed and used under the terms of the GNU GPL.
 *	*NB* No code or data-tables from Powertweak may be used in any profit
 *	     making application (regardless of OS) without express permission.
 *
 * TS: 4
 */

#include "parse_pci.c"

void tune(char *configfile, size_t buffsize);
//void tune(char *configfile, char *endconfig);
//void parse_vendor(char *configfile, char *endconfig);
void parse_vendor(char *configfile, size_t buffsize);

/*
 This routine starts at the top of the config file, and looks for a
 vendor descriptor equal to the vendor of the current device we are
 trying to tune.
 */

void tune(char *configfile, size_t buffsize)
{
    word c_vendor;
    char *conf;
    char *endptr;

    conf=configfile;
    endptr=conf+buffsize;

    while (conf<endptr) {

	switch (*conf) {

	    case '!' :	/* Special commands */
			conf++;
			if (*conf=='.')
			    goto tune_out;
			break;
	
	    case '#' :	/* Skip over comments */
			SkipComment(&conf);
			break;

	    case 'V' :	conf++;
			if (*conf == ':') {
			    conf++;
	    		    c_vendor = strtol(conf, (char **)&conf+4, 16);

			    if (dev->vendor_id == c_vendor) {
	    			parse_vendor(conf, buffsize);
				goto tune_out;
			    }
			    conf++;
			}
			break;

	    default :	conf++;
			continue;
	}
    }
tune_out:
}


/*
 This routine gets passed a pointer to the section in the config file
 that relates to the vendor of the current device.
 We now scan through all the known devices in the config file looking
 for a match with the current device.
 */

void parse_vendor(char *configfile, size_t buffsize)
{
    char namebuf[128];
    unsigned int c_device;
    FILE *logfile=0;
    char *conf;
    char *endptr;

    conf=configfile;
    endptr=conf+buffsize;

    while (conf < endptr) {

	switch (*conf) {
	    case '!' :	/* Special commands */
			conf++;
			if (*conf=='.') {
			    goto out;
			}
			break;

	    case '#' :	SkipComment(&conf);
			break;

	    case ']' :	goto out;

	    case 'D' :
			if (strncmp (conf, "D:", 2) == 0) {
			    conf+=2;
			    c_device = strtol(conf, (char **)&conf+4, 16);
//			    printf("Seeking %x Found D:%x\n", dev->device_id, c_device);

			    if (dev->device_id == c_device) {
//				printf("Found Device!!\n");

				if (open_log_file(&logfile)==TRUE) {
				    fprintf(logfile, "\tFound [%.4x:%.4x] %s\n",
					dev->vendor_id, dev->device_id,
					pci_lookup_name(pacc, namebuf, sizeof(namebuf),
					PCI_LOOKUP_VENDOR | PCI_LOOKUP_DEVICE,
					dev->vendor_id, dev->device_id));
				    close_log_file(&logfile);
				}
				tune_device(conf+6);
				goto out;
			    }
			}
			break;

	    default :	conf++;
			continue;
	}
	conf++;
    }
out:
}


/*
  Optimise_PCI()
  This gets called from main() if PCI optimisation is enabled.
*/

void Optimise_PCI(void)
{
    FILE *filehandle, *logfile;
    size_t buffersize;
    char *configfile=0;
    struct stat fileinfostruct;
    const char fname[]="/usr/share/powertweak/pci.tweaks";


    /* Get file size */
    if (stat(fname,&fileinfostruct) != 0) {
	fprintf (stderr, "File error while opening %s.\n", fname);
        pci_cleanup(pacc);
	exit(EXIT_FAILURE);
    }
    buffersize = fileinfostruct.st_size;

    /* Allocate memory to read file into */
    configfile = malloc(buffersize);

    if (configfile == NULL) {
	fprintf (stderr, "Couldn't malloc %d bytes.\n", buffersize);
	pci_cleanup(pacc);
	exit(EXIT_FAILURE);
    }

    if ((filehandle = fopen (fname, "rb")) == NULL) {
	fprintf (stderr, "Error opening input file : %s\t%s\n",
	    fname, sys_errlist[errno]);
	free(configfile);
	pci_cleanup(pacc);
    	exit(EXIT_FAILURE);
    }

    if (fread (configfile, buffersize, 1, filehandle) != 1) {
	fprintf (stderr, "Error reading from input file : %s\t%s\n",
	    fname, sys_errlist[errno]);
	if (fclose (filehandle) != 0) {
	    fprintf (stderr, "Error closing input file : %s\t%s\n",
		fname, sys_errlist[errno]);
	}
	free(configfile);
	pci_cleanup(pacc);
    	exit(EXIT_FAILURE);
    }

    if (fclose (filehandle) != 0) {
	fprintf (stderr, "Error closing input file : %s\t%s\n",
	    fname, sys_errlist[errno]);
    }

    pci_scan_bus(pacc);
    dev = pacc->devices;


    while (dev) {
	if (open_log_file(&logfile)==TRUE) {
	    fprintf(logfile, "\tSEEK: [%.4x:%.4x]\n",
		dev->vendor_id, dev->device_id);
	    close_log_file(&logfile);
	}

	tune(configfile, buffersize);
	dev = dev->next;
    }

    pci_cleanup(pacc);
    free(configfile);
}
