/*
 *	$Id: powertweak.c,v 1.0.0 1999/6/27 20:34:33 dj Exp $
 *
 *	Linux PCI Utilities -- Tune PCI devices.
 *
 *	Copyright (c) 1999 Dave Jones <dave@powertweak.com>
 *	contains some code by Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 *
 *	Can be freely distributed and used under the terms of the GNU GPL.
 *	*NB* No code or data-tables from Powertweak may be used in any profit
 *	     making application (regardless of OS) without express permission.
 *
 *	DISCLAIMER:
 *		The authors cannot be held responsible for any loss incurred
 *		either directly, or indirectly as a result of running this
 *		utility. Like any other program, there may be bugs lurking
 *		someplace that haven't yet been noticed. As this program
 *		is tweaking low-level chipset registers, anything could happen.
 * TS: 4
 */

#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <syslog.h>

#include <errno.h>
#include <sys/stat.h>

#include "pciutils.h"

/*#define DEBUG*/

#define FALSE 0
#define TRUE 1

#define POWERTWEAK_VERSION "0.1.0"

/* Options */
int diff=0;			/* Only display changed regs */
int quiet=0;			/* Completely silent. */
int verbose=0;			/* Show detailed information */
int hex=0;			/* Show values of registers.*/

/* Communication with libpci */
struct pci_access *pacc;
struct pci_dev *dev;

/* Prototypes */
void Optimise_CPU(void);
void Optimise_PCI(void);

#include "log.c"
#include "CPU/cpu.c"
#include "PCI/pci.c"

/**************************
    Main
 **************************/

int main(int argc, char **argv)
{
    FILE *logfile=0;
    int i;
    const char versionstring[]= "Powertweak-Linux v" POWERTWEAK_VERSION "\n(c) 1999 Dave Jones. [dave@powertweak.com]\n";
    const char TuningString1[]="[ 33%%][...       ][CPU]\r";
    const char TuningString2[]="[ 66%%][......    ][PCI]\r";
    const char TuningDone[]=   "[DONE][System tuned]    \n";
    const char options[] = "" GENERIC_OPTIONS ;
    const char help_msg[] = "\
Usage: Powertweak [<switches>]\n"\
GENERIC_HELP
;


    if (argc == 2 && !strncasecmp(argv[1], "--version", 9)) {
	printf("v" POWERTWEAK_VERSION "\n");
	exit (EXIT_SUCCESS);
    }

    if (geteuid()) {
	fprintf(stderr, "Powertweak can only be run by root.\n");
	exit(EXIT_FAILURE);

    } else {

        if (quiet == 0)
    	    printf(versionstring);
	    syslog(LOG_INFO, "%s", versionstring);
	    if (open_log_file(&logfile)==TRUE) {
		fprintf(logfile, "%s", versionstring);
		close_log_file(&logfile);
	    }

        pacc = pci_alloc();
	pci_init(pacc);
        pacc->error = die;
    
        while (i = getopt(argc, argv, options) != -1)
	switch (i) {

	    default:	if (parse_generic_option(i, pacc, optarg))
			    break;
	    bad:	fprintf(stderr, help_msg, pacc->id_file_name);
			pci_cleanup(pacc);
			exit(EXIT_FAILURE);
	}

	if (optind < argc)
	    goto bad;


// FIXME: A loop would be preferred here. - DJ.

	printf ("%s", TuningString1);
	if (open_log_file(&logfile)==TRUE) {
	    fprintf(logfile, "%s\n", TuningString1);
	    close_log_file(&logfile);
	}
	Optimise_CPU();


	printf ("%s", TuningString2);
	if (open_log_file(&logfile)==TRUE) {
	    fprintf(logfile, "%s\n", TuningString2);
	    close_log_file(&logfile);
	}
	Optimise_PCI();

	if (open_log_file(&logfile)==TRUE) {
	    fprintf(logfile, "%s", TuningDone);
	    close_log_file(&logfile);
	}
	printf ("%s", TuningDone);


	exit(EXIT_SUCCESS);
    }
}
